@@ -2,15 +2,18 @@ use std::fmt;
 use std::cell::Cell;
 use std::hash::{Hash, Hasher};
 use std::cmp::{PartialEq, Eq};
+use static_assertions::assert_eq_size;
 use crate::language::{IntType, FloatType};
 use crate::runtime::types::Type;
 use crate::runtime::types::metatable::Metatable;
 use crate::runtime::function::{Function, NativeFunction, Call, Invoke};
 use crate::runtime::types::primitive::*;
 use crate::runtime::strings::StringSymbol;
-use crate::runtime::gc::{GC, GCArray, GCTrace};
+use crate::runtime::gc::{GC, GCTrace};
 use crate::runtime::errors::{ExecResult, RuntimeError, ErrorKind};
 
+#[cfg(target_arch = "x86_64")]
+assert_eq_size!(Variant, [u8; 16]);
 
 // Fundamental data value type
 #[derive(Clone, Copy)]
@@ -24,7 +27,7 @@ pub enum Variant {
     Float(FloatType),
     String(StringSymbol),
     
-    Tuple(GCArray<Variant>),
+    Tuple(GC<Box<[Variant]>>), // TODO: stop using Box when DST support is stabilized
     Function(GC<Function>),
     NativeFunction(GC<NativeFunction>),
 }
@@ -77,22 +80,12 @@ impl Variant {
         Some(value)
     }
     
-    pub fn can_hash(&self) -> bool {
-        match self {
-            Self::Float(..) => false,
-            Self::Tuple(items) => items.iter().all(|item| item.can_hash()),
-            // TODO Objects - check metatabale for __hash
-            _ => true
-        }
-        
-    }
-    
     pub fn invoke(&self, args: &[Variant]) -> ExecResult<Call> {
         match self {
             Self::Function(fun) => fun.invoke(args),
             Self::NativeFunction(fun) => fun.invoke(args),
             
-            _ => Err(ErrorKind::NotCallable(self.clone()).into())
+            _ => Err(ErrorKind::NotCallable(*self).into())
         }
     }
     
@@ -130,25 +123,51 @@ impl From<&str> for Variant {
 
 impl From<Box<[Variant]>> for Variant {
     fn from(items: Box<[Variant]>) -> Self {
-        Self::Tuple(GCArray::from_boxed_slice(items))
+        Self::Tuple(GC::new(items))
     }
 }
 
 impl From<Function> for Variant {
     fn from(func: Function) -> Self {
-        Self::Function(GC::allocate(func))
+        Self::Function(GC::new(func))
     }
 }
 
 impl From<NativeFunction> for Variant {
     fn from(func: NativeFunction) -> Self {
-        Self::NativeFunction(GC::allocate(func))
+        Self::NativeFunction(GC::new(func))
     }
 }
 
-impl GCTrace for Variant { }
+unsafe impl GCTrace for Variant {
+    #[inline]
+    fn trace(&self) {
+        match self {
+            Self::Tuple(items) => items.mark_trace(),
+            Self::Function(fun) => fun.mark_trace(),
+            Self::NativeFunction(fun) => fun.mark_trace(),
+            _ => { },
+        };
+    }
+}
 
-impl GCTrace for Cell<Variant> { }
+unsafe impl GCTrace for Cell<Variant> {
+    fn trace(&self) {
+        self.get().trace()
+    }
+}
+
+unsafe impl GCTrace for Box<[Variant]> {
+    fn trace(&self) {
+        for item in self.iter() {
+            item.trace();
+        }
+    }
+    
+    fn size_hint(&self) -> usize {
+        std::mem::size_of::<Variant>() * self.len()
+    }
+}
 
 
 // extract the GC handle for GC'd types
@@ -157,7 +176,7 @@ impl TryFrom<Variant> for GC<dyn GCTrace> {
     
     fn try_from(value: Variant) -> Result<Self, Self::Error> {
         match value {
-            Variant::Tuple(tuple) => Ok(tuple.handle()),
+            Variant::Tuple(tuple) => Ok(tuple.into()),
             Variant::Function(fun) => Ok(fun.into()),
             Variant::NativeFunction(fun) => Ok(fun.into()),
             _ => Err(value),
@@ -166,8 +185,44 @@ impl TryFrom<Variant> for GC<dyn GCTrace> {
 }
 
 
-// Not all Variants are hashable, so there is a separate type to handle that
+// Not all Variants are hashable
+impl Variant {
+    pub fn can_hash(&self) -> bool {
+        struct DummyHasher();
+        impl Hasher for DummyHasher {
+            fn finish(&self) -> u64 { 0 }
+            fn write(&mut self, _: &[u8]) { }
+        }
+        self.try_hash(&mut DummyHasher()).is_ok()
+    }
+    
+    fn try_hash<H: Hasher>(&self, state: &mut H) -> ExecResult<()> {
+        let discr = std::mem::discriminant(self);
+        
+        match self {
+            Self::Nil | Self::BoolTrue | Self::BoolFalse | Self::EmptyTuple 
+                => discr.hash(state),
+            
+            Self::Integer(value) => (discr, value).hash(state),
+            Self::String(value) => (discr, value).hash(state),
+            Self::Function(fun) => (discr, fun).hash(state),
+            Self::NativeFunction(fun) => (discr, fun).hash(state),
+            Self::Tuple(items) => {
+                discr.hash(state); // also prevent prefix collisions
+                for item in items.iter() {
+                    item.try_hash(state)?;
+                }
+            },
+            
+            _ => return Err(ErrorKind::UnhashableValue(*self).into()),
+        }
+        Ok(())
+    }
+    
+}
 
+
+/// Wrapper for variant that dynamically ensures hashability
 #[derive(Clone)]
 pub struct VariantKey<'a>(&'a Variant);
 
@@ -175,25 +230,15 @@ impl<'a> TryFrom<&'a Variant> for VariantKey<'a> {
     type Error = RuntimeError;
     fn try_from(value: &'a Variant) -> ExecResult<Self> {
         if !value.can_hash() {
-            return Err(ErrorKind::UnhashableValue(value.clone()).into());
+            return Err(ErrorKind::UnhashableValue(*value).into());
         }
         Ok(Self(value))
     }
 }
 
 impl Hash for VariantKey<'_> {
-    fn hash<H>(&self, state: &mut H) where H: Hasher {
-        debug_assert!(self.0.can_hash());
-        
-        let discriminant = std::mem::discriminant(self.0);
-        discriminant.hash(state);
-        
-        match self.0 {
-            Variant::Integer(value) => value.hash(state),
-            Variant::String(strkey) => strkey.hash(state),
-            // TODO objects - get the result of __hash and hash it
-            _ => { }
-        }
+    fn hash<H: Hasher>(&self, state: &mut H) {
+        self.0.try_hash(state).unwrap()
     }
 }
 
@@ -214,7 +259,8 @@ impl<'s> PartialEq for VariantKey<'_> {
                 .all(|(a, b)| VariantKey(a) == VariantKey(b))
             },
             
-            // TODO objects, use __eq
+            (Variant::Function(a), Variant::Function(b)) => GC::ptr_eq(a, b),
+            (Variant::NativeFunction(a), Variant::NativeFunction(b)) => GC::ptr_eq(a, b),
             
             _ => false,
         }
